@@ -84,6 +84,9 @@ struct _GskCurveClass
                                                          const graphene_point_t *point);
   float                         (* get_length_to)       (const GskCurve         *curve,
                                                          float                   t);
+  float                         (* get_at_length)       (const GskCurve         *curve,
+                                                         float                   distance,
+                                                         float                   epsilon);
 };
 
 /* {{{ Utilities */
@@ -208,6 +211,41 @@ get_length_by_approximation (const GskCurve *curve,
   return z * sum;
 }
 
+/* Compute the inverse of the arclength using bisection,
+ * to a given precision
+ */
+static float
+get_t_by_bisection (const GskCurve *curve,
+                    float           length,
+                    float           epsilon)
+{
+  float t1, t2, t, l;
+  GskCurve c1;
+
+  g_assert (epsilon >= FLT_EPSILON);
+
+  t1 = 0;
+  t2 = 1;
+
+  while (t1 < t2)
+    {
+      t = (t1 + t2) / 2;
+      if (t == t1 || t == t2)
+        break;
+
+      gsk_curve_split (curve, t, &c1, NULL);
+
+      l = gsk_curve_get_length (&c1);
+      if (fabsf (length - l) < epsilon)
+        break;
+      else if (l < length)
+        t1 = t;
+      else
+        t2 = t;
+    }
+
+  return t;
+}
 /* }}} */
 /* {{{ Line */
 
@@ -426,6 +464,23 @@ gsk_line_curve_get_length_to (const GskCurve *curve,
   return t * graphene_point_distance (&pts[0], &pts[1], NULL, NULL);
 }
 
+static float
+gsk_line_curve_get_at_length (const GskCurve *curve,
+                              float           distance,
+                              float           epsilon)
+{
+  const GskLineCurve *self = &curve->line;
+  const graphene_point_t *pts = self->points;
+  float length;
+
+  length = graphene_point_distance (&pts[0], &pts[1], NULL, NULL);
+
+  if (length == 0)
+    return 0;
+
+  return CLAMP (distance / length, 0, 1);
+}
+
 static const GskCurveClass GSK_LINE_CURVE_CLASS = {
   gsk_line_curve_init,
   gsk_line_curve_init_foreach,
@@ -448,6 +503,7 @@ static const GskCurveClass GSK_LINE_CURVE_CLASS = {
   gsk_line_curve_get_derivative_at,
   gsk_line_curve_get_crossing,
   gsk_line_curve_get_length_to,
+  gsk_line_curve_get_at_length,
 };
 
 /* }}} */
@@ -721,6 +777,19 @@ gsk_quad_curve_decompose_curve (const GskCurve       *curve,
 
   if (flags & GSK_PATH_FOREACH_ALLOW_QUAD)
     return add_curve_func (GSK_PATH_QUAD, self->points, 3, 0.f, user_data);
+  else if (graphene_point_equal (&curve->conic.points[0], &curve->conic.points[1]) ||
+           graphene_point_equal (&curve->conic.points[1], &curve->conic.points[2]))
+    {
+      if (!graphene_point_equal (&curve->conic.points[0], &curve->conic.points[2]))
+        return add_curve_func (GSK_PATH_LINE,
+                               (graphene_point_t[2]) {
+                                 curve->conic.points[0],
+                                 curve->conic.points[2],
+                               },
+                               2, 0.f, user_data);
+      else
+        return TRUE;
+    }
   else if (flags & GSK_PATH_FOREACH_ALLOW_CUBIC)
     {
       GskCurve c;
@@ -828,6 +897,14 @@ gsk_quad_curve_get_length_to (const GskCurve *curve,
   return get_length_by_approximation (curve, t);
 }
 
+static float
+gsk_quad_curve_get_at_length (const GskCurve *curve,
+                              float           t,
+                              float           epsilon)
+{
+  return get_t_by_bisection (curve, t, epsilon);
+}
+
 static const GskCurveClass GSK_QUAD_CURVE_CLASS = {
   gsk_quad_curve_init,
   gsk_quad_curve_init_foreach,
@@ -850,9 +927,10 @@ static const GskCurveClass GSK_QUAD_CURVE_CLASS = {
   gsk_quad_curve_get_derivative_at,
   gsk_quad_curve_get_crossing,
   gsk_quad_curve_get_length_to,
+  gsk_quad_curve_get_at_length,
 };
 
-/* }}} */ 
+/* }}} */
 /* {{{ Cubic */
 
 static void
@@ -1298,6 +1376,14 @@ gsk_cubic_curve_get_length_to (const GskCurve *curve,
   return get_length_by_approximation (curve, t);
 }
 
+static float
+gsk_cubic_curve_get_at_length (const GskCurve *curve,
+                               float           t,
+                               float           epsilon)
+{
+  return get_t_by_bisection (curve, t, epsilon);
+}
+
 static const GskCurveClass GSK_CUBIC_CURVE_CLASS = {
   gsk_cubic_curve_init,
   gsk_cubic_curve_init_foreach,
@@ -1320,6 +1406,7 @@ static const GskCurveClass GSK_CUBIC_CURVE_CLASS = {
   gsk_cubic_curve_get_derivative_at,
   gsk_cubic_curve_get_crossing,
   gsk_cubic_curve_get_length_to,
+  gsk_cubic_curve_get_at_length,
 };
 
  /*  }}} */
@@ -1853,7 +1940,20 @@ gsk_conic_curve_decompose_or_add (const GskCurve       *curve,
                                   GskCurveAddCurveFunc  add_curve_func,
                                   gpointer              user_data)
 {
-  if (gsk_conic_is_close_to_cubic (curve, cubic, tolerance))
+  if (graphene_point_equal (&curve->conic.points[0], &curve->conic.points[1]) ||
+      graphene_point_equal (&curve->conic.points[1], &curve->conic.points[3]))
+    {
+      if (!graphene_point_equal (&curve->conic.points[0], &curve->conic.points[3]))
+        return add_curve_func (GSK_PATH_LINE,
+                               (graphene_point_t[2]) {
+                                 curve->conic.points[0],
+                                 curve->conic.points[3],
+                               },
+                               2, 0.f, user_data);
+      else
+        return TRUE;
+    }
+  else if (gsk_conic_is_close_to_cubic (curve, cubic, tolerance))
     return add_curve_func (GSK_PATH_CUBIC, cubic->cubic.points, 4, 0.f, user_data);
   else
     {
@@ -1895,7 +1995,7 @@ gsk_conic_curve_decompose_curve (const GskCurve       *curve,
       return gsk_conic_curve_decompose_or_add (curve, &c, tolerance, add_curve_func, user_data);
     }
 
-  /* FIXME: Quadratic (or conic?) approximation */
+  /* FIXME: Quadratic approximation */
   return gsk_conic_curve_decompose (curve,
                                     tolerance,
                                     gsk_curve_add_line_cb,
@@ -1992,6 +2092,14 @@ gsk_conic_curve_get_length_to (const GskCurve *curve,
   return get_length_by_approximation (curve, t);
 }
 
+static float
+gsk_conic_curve_get_at_length (const GskCurve *curve,
+                               float           t,
+                               float           epsilon)
+{
+  return get_t_by_bisection (curve, t, epsilon);
+}
+
 static const GskCurveClass GSK_CONIC_CURVE_CLASS = {
   gsk_conic_curve_init,
   gsk_conic_curve_init_foreach,
@@ -2014,6 +2122,7 @@ static const GskCurveClass GSK_CONIC_CURVE_CLASS = {
   gsk_conic_curve_get_derivative_at,
   gsk_conic_curve_get_crossing,
   gsk_conic_curve_get_length_to,
+  gsk_conic_curve_get_at_length,
 };
 
 /*  }}} */
@@ -2363,32 +2472,7 @@ gsk_curve_at_length (const GskCurve *curve,
                      float           length,
                      float           epsilon)
 {
-  float t1, t2, t, l;
-  GskCurve c1;
-
-  g_assert (epsilon >= FLT_EPSILON);
-
-  t1 = 0;
-  t2 = 1;
-
-  while (t1 < t2)
-    {
-      t = (t1 + t2) / 2;
-      if (t == t1 || t == t2)
-        break;
-
-      gsk_curve_split (curve, t, &c1, NULL);
-
-      l = gsk_curve_get_length (&c1);
-      if (fabsf (length - l) < epsilon)
-        break;
-      else if (l < length)
-        t1 = t;
-      else
-        t2 = t;
-    }
-
-  return t;
+  return get_class (curve->op)->get_at_length (curve, length, epsilon);
 }
 
 static inline void
@@ -2497,6 +2581,57 @@ gsk_curve_get_curvature_points (const GskCurve *curve,
   return filter_allowable (t, n);
 }
 
+/* Find cusps inside the open interval from 0 to 1.
+ *
+ * According to Stone & deRose, A Geometric Characterization
+ * of Parametric Cubic curves, a necessary and sufficient
+ * condition is that the first derivative vanishes.
+ */
+int
+gsk_curve_get_cusps (const GskCurve *curve,
+                     float           t[2])
+{
+  const graphene_point_t *pts = curve->cubic.points;
+  graphene_point_t p[3];
+  float ax, bx, cx;
+  float ay, by, cy;
+  float tx[3];
+  int nx;
+  int n = 0;
+
+  if (curve->op != GSK_PATH_CUBIC)
+    return 0;
+
+  p[0].x = 3 * (pts[1].x - pts[0].x);
+  p[0].y = 3 * (pts[1].y - pts[0].y);
+  p[1].x = 3 * (pts[2].x - pts[1].x);
+  p[1].y = 3 * (pts[2].y - pts[1].y);
+  p[2].x = 3 * (pts[3].x - pts[2].x);
+  p[2].y = 3 * (pts[3].y - pts[2].y);
+
+  ax = p[0].x - 2 * p[1].x + p[2].x;
+  bx = - 2 * p[0].x + 2 * p[1].x;
+  cx = p[0].x;
+
+  nx = solve_quadratic (ax, bx, cx, tx);
+  nx = filter_allowable (tx, nx);
+
+  ay = p[0].y - 2 * p[1].y + p[2].y;
+  by = - 2 * p[0].y + 2 * p[1].y;
+  cy = p[0].y;
+
+  for (int i = 0; i < nx; i++)
+    {
+      float ti = tx[i];
+
+      if (0 < ti && ti < 1 &&
+          fabsf (ay * ti * ti + by * ti + cy) < 0.001)
+        t[n++] = ti;
+    }
+
+  return n;
+}
+
 /* }}} */
 
 /* vim:set foldmethod=marker expandtab: */
